import Argument from './Argument.js';
import TreeRegexp from './TreeRegexp.js';
import ParameterType from './ParameterType.js';
export default class RegularExpression {
    constructor(regexp, parameterTypeRegistry) {
        this.regexp = regexp;
        this.parameterTypeRegistry = parameterTypeRegistry;
        this.treeRegexp = new TreeRegexp(regexp);
    }
    match(text) {
        const parameterTypes = this.treeRegexp.groupBuilder.children.map((groupBuilder) => {
            const parameterTypeRegexp = groupBuilder.source;
            return (this.parameterTypeRegistry.lookupByRegexp(parameterTypeRegexp, this.regexp, text) ||
                new ParameterType(null, parameterTypeRegexp, String, (s) => (s === undefined ? null : s), false, false));
        });
        return Argument.build(this.treeRegexp, text, parameterTypes);
    }
    get source() {
        return this.regexp.source;
    }
}
//# sourceMappingURL=RegularExpression.js.map